// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt (
	avst_clk				,
	avmm_clk				,
	rst_n					,

	avms_address_i			,
	avms_byteenable_i		,
	avms_read_i				,
	avms_write_i			,
	avms_readdatavalid_o	,
	avms_readdata_o			,
	avms_writedata_i		,
	avms_waitrequest_o		,

	int_o					,

	avss_valid_i			,
	avss_startofpacket_i	,
	avss_endofpacket_i		,
	avss_data_i				,
	avss_mod_i				,
	avss_err_i				,
	avss_ready_o			,

	avmm_read_o				,
	avmm_write_o			,
	avmm_waitrequest_i		,
	avmm_address_o			,
	avmm_burstcount_o		,
	avmm_byteenable_o		,
	avmm_writedata_o		,
	avmm_readdatavalid_i	,
	avmm_readdata_i			
) ;


// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	AVMS_ADR_BIT	= 6 ;
	parameter	AVMS_DATA_BIT	= 32 ;
	parameter	AVMS_BE_BIT		= AVMS_DATA_BIT / 8 ;

	parameter	ST_DATA_BIT		= 32 ;
	parameter	FIFO0_DATA_BIT	= ST_DATA_BIT * 2 + 2 ;	// FIFO data bit width
	parameter	INT_ST_DATA_BIT	= ST_DATA_BIT * 2 ;
	parameter	FIFO2_DATA_BIT	= INT_ST_DATA_BIT * 2 ;	// FIFO data bit width
	parameter	AVMM_ADR_BIT	= 27 ;
	parameter	AVMM_DATA_BIT	= 128 ;
	parameter	AVMM_BURST_BIT	= 8 ;
	parameter	AVMM_BE_BIT		= AVMM_DATA_BIT / 8 ;

	parameter	MAC_ADR_BIT		= 48 ;
	parameter	TAG_BIT			= 32 ;
	parameter	TYPE_BIT		= 16 ;
	parameter	FRM_NUM_BIT		= 16 ;
	parameter	PACKET_NUM_BIT	= 32 ;
	parameter	T_STP_SEC_BIT	= 48 ;
	parameter	T_STP_NS_BIT	= 32 ;
	
	parameter	PACKET_SIZE_BIT	= 11 ;

// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire								avst_clk				;
	input	wire								avmm_clk				;
	input	wire								rst_n					;

	input	wire	[    AVMS_ADR_BIT-1 : 0 ]	avms_address_i			;
	input	wire	[ AVMS_DATA_BIT/8-1 : 0 ]	avms_byteenable_i		;
	input	wire								avms_read_i				;
	input	wire								avms_write_i			;
	output	wire								avms_readdatavalid_o	;
	output	wire	[   AVMS_DATA_BIT-1 : 0 ]	avms_readdata_o			;
	input	wire	[   AVMS_DATA_BIT-1 : 0 ]	avms_writedata_i		;
	output	wire								avms_waitrequest_o		;

	output	wire								int_o					;

	input	wire								avss_valid_i			;
	input	wire								avss_startofpacket_i	;
	input	wire								avss_endofpacket_i		;
	input	wire	[     ST_DATA_BIT-1 : 0 ]	avss_data_i				;
	input	wire	[                 1 : 0 ]	avss_mod_i				;
	input	wire	[                 5 : 0 ]	avss_err_i				;
	output	wire								avss_ready_o			;

	output	wire								avmm_read_o				;
	output	wire								avmm_write_o			;
	input	wire								avmm_waitrequest_i		;
	output	wire	[    AVMM_ADR_BIT-1 : 0 ]	avmm_address_o			;
	output	wire	[  AVMM_BURST_BIT-1 : 0 ]	avmm_burstcount_o		;
	output	wire	[ AVMM_DATA_BIT/8-1 : 0 ]	avmm_byteenable_o		;
	output	wire	[   AVMM_DATA_BIT-1 : 0 ]	avmm_writedata_o		;
	input	wire								avmm_readdatavalid_i	;
	input	wire	[   AVMM_DATA_BIT-1 : 0 ]	avmm_readdata_i			;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	wire								csr_srst					;
	wire	[ PACKET_SIZE_BIT-1 : 0 ]	csr_packet_size				;
	wire	[     MAC_ADR_BIT-1 : 0 ]	csr_mac_adr					;
	wire	[  AVMM_ADR_BIT+4-1 : 0 ]	csr_acc_adr					;
	wire								csr_adr_valid				;

	wire								u0_fifo_wen					;
	wire								u0_fifo_ren					;
	wire	[  FIFO0_DATA_BIT-1 : 0 ]	u0_fifo_wdata				;
	wire	[  FIFO0_DATA_BIT-1 : 0 ]	u0_fifo_rdata				;

	wire								u0_fifo_aempty				;
	wire								u0_fifo_empty				;
	wire								u0_fifo_afull				;
	wire								u0_fifo_full				;

	wire								pfifo_rd_valid_o			;
	wire								pfifo_rd_startofpacket_o	;
	wire								pfifo_rd_endofpacket_o		;
	wire	[ INT_ST_DATA_BIT-1 : 0 ]	pfifo_rd_data_o				;
	wire								pfifo_rd_ready_i			;

	wire	[     FRM_NUM_BIT-1 : 0 ]	pkt_frame_number			;
	wire	[  PACKET_NUM_BIT-1 : 0 ]	pkt_packet_number			;
	wire	[  PACKET_NUM_BIT-1 : 0 ]	pkt_total_packet_count		;
	wire	[   T_STP_SEC_BIT-1 : 0 ]	pkt_time_stamp_sec			;
	wire	[    T_STP_NS_BIT-1 : 0 ]	pkt_time_stamp_ns			;
	wire								pkt_fifo_wt_req				;
	wire								pkt_fifo_wt_start			;
	wire								pkt_fifo_wt_end				;

	wire								u1_fifo_wen					;
	wire								u1_fifo_ren					;
	wire	[  FIFO2_DATA_BIT-1 : 0 ]	u1_fifo_rdata				;
	wire								u1_fifo_aempty				;
	wire								u1_fifo_empty				;
	wire								u1_fifo_afull				;
	wire								u1_fifo_full				;

	wire								u2_fifo_wen					;
	wire								u2_fifo_ren					;
	wire	[  FIFO2_DATA_BIT-1 : 0 ]	u2_fifo_rdata				;
	wire								u2_fifo_aempty				;
	wire								u2_fifo_empty				;
	wire								u2_fifo_afull				;
	wire								u2_fifo_full				;

	wire	[  FIFO2_DATA_BIT-1 : 0 ]	u12_fifo_wdata				;

	wire	[                 1 : 0 ]	fifo_bank_wt_sel			;
	wire	[  AVMM_ADR_BIT+4-1 : 0 ]	fifo_bank_acc_adr			;
	wire								mem_if_fifo_rd_end			;
	wire	[                 1 : 0 ]	fifo_bank_rd_sel			;


// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	avb_frm_wt_csr 
		# (
			.AVMS_ADR_BIT		( AVMS_ADR_BIT		) ,
			.AVMS_DATA_BIT		( AVMS_DATA_BIT		) ,
			.AVMS_BE_BIT		( AVMS_BE_BIT		) ,
			.AVMM_ADR_BIT		( AVMM_ADR_BIT		) ,
			.MAC_ADR_BIT		( MAC_ADR_BIT		) ,
			.TYPE_BIT			( TYPE_BIT			) ,
			.FRM_NUM_BIT		( FRM_NUM_BIT		) ,
			.PACKET_NUM_BIT		( PACKET_NUM_BIT	) ,
			.T_STP_SEC_BIT		( T_STP_SEC_BIT		) ,
			.T_STP_NS_BIT		( T_STP_NS_BIT		) ,
			.PACKET_SIZE_BIT	( PACKET_SIZE_BIT	)
		)
		u0_csr (
			.clk					( avmm_clk					) ,
			.rst_n					( rst_n						) ,

			.address_i				( avms_address_i			) ,
			.byteenable_i			( avms_byteenable_i			) ,
			.read_i					( avms_read_i				) ,
			.write_i				( avms_write_i				) ,
			.readdatavalid_o		( avms_readdatavalid_o		) ,
			.readdata_o				( avms_readdata_o			) ,
			.writedata_i			( avms_writedata_i			) ,
			.waitrequest_o			( avms_waitrequest_o		) ,

			.frame_number_i			( pkt_frame_number			) ,
			.packet_number_i		( pkt_packet_number			) ,
			.total_packet_count_i	( pkt_total_packet_count	) ,
			.time_stamp_sec_i		( pkt_time_stamp_sec		) ,
			.time_stamp_ns_i		( pkt_time_stamp_ns			) ,
			.fifo_wt_start_i		( pkt_fifo_wt_start			) ,
			.fifo_wt_end_i			( pkt_fifo_wt_end			) ,
			.mem_wt_end_i			( mem_if_fifo_rd_end		) ,

			.srst_o					( csr_srst					) ,
			.packet_size_o			( csr_packet_size			) ,
			.src_mac_adr_o			( csr_mac_adr				) ,
			.acc_adr_o				( csr_acc_adr				) ,
			.acc_adr_valid_o		( csr_adr_valid				) ,

			.rx_frame_end_int_o		( int_o						)
		) ;

	avb_frm_wt_st_in 
		# (
			.ST_DATA_BIT	( ST_DATA_BIT	) 
		)
		u0_st_in (
			.clk				( avst_clk				) ,
			.rst_n				( rst_n					) ,
			.srst				( csr_srst				) ,

			.valid_i			( avss_valid_i			) ,
			.startofpacket_i	( avss_startofpacket_i	) ,
			.endofpacket_i		( avss_endofpacket_i	) ,
			.data_i				( avss_data_i			) ,
			.ready_o			( avss_ready_o			) ,

			.fifo_full_i		( u0_fifo_full			) ,
			.fifo_afull_i		( u0_fifo_afull			) ,
			.fifo_we_o			( u0_fifo_wen			) ,
			.fifo_wdata_o		( u0_fifo_wdata			) 
		) ;

	avb_frm_wt_async_fifo
		# (
			.FIFO_DEPTH		( 10				) ,
			.DATA_BIT_WIDTH	( FIFO0_DATA_BIT	) ,
			.WATERMARK_W	( 1 				) ,
			.WATERMARK_R	( 1					)
		)
		u0_async_fifo (
			.wclk		( avst_clk				) ,
			.rclk		( avmm_clk				) ,
			.rst_n		( rst_n					) ,
			.srst		( csr_srst				) ,

			.wen_i		( u0_fifo_wen			) ,
			.ren_i		( u0_fifo_ren			) ,
			.wdata_i	( u0_fifo_wdata			) ,
			.rdata_o	( u0_fifo_rdata			) ,

			.a_empty_o	( u0_fifo_aempty		) ,
			.empty_o	( u0_fifo_empty			) ,
			.a_full_o	( u0_fifo_afull			) ,
			.full_o		( u0_fifo_full			) 
		) ;

	avb_frm_wt_pfifo_rd 
		# (
			.ST_DATA_BIT	( INT_ST_DATA_BIT	) 
		)
		u0_pfifo_rd (
			.clk				( avmm_clk					) ,
			.rst_n				( rst_n						) ,
			.srst				( csr_srst					) ,

			.fifo_empty_i		( u0_fifo_empty				) ,
			.fifo_aempty_i		( u0_fifo_aempty			) ,
			.fifo_re_o			( u0_fifo_ren				) ,
			.fifo_rdata_i		( u0_fifo_rdata				) ,

			.valid_o			( pfifo_rd_valid_o			) ,
			.startofpacket_o	( pfifo_rd_startofpacket_o	) ,
			.endofpacket_o		( pfifo_rd_endofpacket_o	) ,
			.data_o				( pfifo_rd_data_o			) ,
			.ready_i			( pfifo_rd_ready_i			) 
		) ;

	avb_frm_wt_pkt_anls 
		# (
			.ST_DATA_BIT	( INT_ST_DATA_BIT	) ,
			.MAC_ADR_BIT	( MAC_ADR_BIT		) ,
			.TAG_BIT		( TAG_BIT			) ,
			.TYPE_BIT		( TYPE_BIT			) ,
			.FRM_NUM_BIT	( FRM_NUM_BIT		) ,
			.PACKET_NUM_BIT	( PACKET_NUM_BIT	) ,
			.T_STP_SEC_BIT	( T_STP_SEC_BIT		) ,
			.T_STP_NS_BIT	( T_STP_NS_BIT		) 
		)
		u0_pkt_anls (
			.clk					( avmm_clk					) ,
			.rst_n					( rst_n						) ,
			.srst					( csr_srst					) ,

			.valid_i				( pfifo_rd_valid_o			) ,
			.startofpacket_i		( pfifo_rd_startofpacket_o	) ,
			.endofpacket_i			( pfifo_rd_endofpacket_o	) ,
			.data_i					( pfifo_rd_data_o			) ,
			.ready_o				( pfifo_rd_ready_i			) ,

			.src_mac_adr_i			( csr_mac_adr				) ,
			.frame_number_o			( pkt_frame_number			) ,
			.packet_number_o		( pkt_packet_number			) ,
			.total_packet_count_o	( pkt_total_packet_count	) ,
			.time_stamp_sec_o		( pkt_time_stamp_sec		) ,
			.time_stamp_ns_o		( pkt_time_stamp_ns			) ,
			.fifo_wt_req_o			( pkt_fifo_wt_req			) ,
			.fifo_wt_start_o		( pkt_fifo_wt_start			) ,
			.fifo_wt_end_o			( pkt_fifo_wt_end			) ,

			.fifo_sel_i				( fifo_bank_wt_sel			) ,
			.fifo0_full_i			( u1_fifo_full				) ,
			.fifo0_afull_i			( u1_fifo_afull				) ,
			.fifo0_we_o				( u1_fifo_wen				) ,
			.fifo1_full_i			( u2_fifo_full				) ,
			.fifo1_afull_i			( u2_fifo_afull				) ,
			.fifo1_we_o				( u2_fifo_wen				) ,
			.fifo_wdata_o			( u12_fifo_wdata			)
		) ;

	avb_frm_wt_async_fifo
		# (
			.FIFO_DEPTH		( 11				) ,
			.DATA_BIT_WIDTH	( FIFO2_DATA_BIT	) ,
			.WATERMARK_W	( 1 				) ,
			.WATERMARK_R	( 1					)
		)
		u1_async_fifo (
			.wclk		( avmm_clk				) ,
			.rclk		( avmm_clk				) ,
			.rst_n		( rst_n					) ,
			.srst		( csr_srst				) ,

			.wen_i		( u1_fifo_wen			) ,
			.ren_i		( u1_fifo_ren			) ,
			.wdata_i	( u12_fifo_wdata		) ,
			.rdata_o	( u1_fifo_rdata			) ,

			.a_empty_o	( u1_fifo_aempty		) ,
			.empty_o	( u1_fifo_empty			) ,
			.a_full_o	( u1_fifo_afull			) ,
			.full_o		( u1_fifo_full			) 
		) ;

	avb_frm_wt_async_fifo
		# (
			.FIFO_DEPTH		( 11				) ,
			.DATA_BIT_WIDTH	( FIFO2_DATA_BIT	) ,
			.WATERMARK_W	( 1 				) ,
			.WATERMARK_R	( 1					)
		)
		u2_async_fifo (
			.wclk		( avmm_clk				) ,
			.rclk		( avmm_clk				) ,
			.rst_n		( rst_n					) ,
			.srst		( csr_srst				) ,

			.wen_i		( u2_fifo_wen			) ,
			.ren_i		( u2_fifo_ren			) ,
			.wdata_i	( u12_fifo_wdata		) ,
			.rdata_o	( u2_fifo_rdata			) ,
                          
			.a_empty_o	( u2_fifo_aempty		) ,
			.empty_o	( u2_fifo_empty			) ,
			.a_full_o	( u2_fifo_afull			) ,
			.full_o		( u2_fifo_full			) 
		) ;

	avb_frm_wt_fifo_bank 
		# (
			.AVMM_ADR_BIT	( AVMM_ADR_BIT + 4	)
		)
		u0_fifo_bank (
			.clk				( avmm_clk				) ,
			.rst_n				( rst_n					) ,
			.srst				( csr_srst				) ,

			.acc_adr_i			( csr_acc_adr			) ,
			.acc_adr_valid_i	( csr_adr_valid			) ,

			.fifo_wt_req_i		( pkt_fifo_wt_req		) ,
			.fifo_wt_end_i		( pkt_fifo_wt_end		) ,
			.fifo_wt_sel_o		( fifo_bank_wt_sel		) ,

			.fifo_rd_end_i		( mem_if_fifo_rd_end	) ,
			.acc_adr_o			( fifo_bank_acc_adr		) ,
			.fifo_rd_sel_o		( fifo_bank_rd_sel		) 
		) ;

	avb_frm_wt_mem_if 
		# (
			.AVM_ADR_BIT	( AVMM_ADR_BIT		) ,
			.AVM_DATA_BIT	( AVMM_DATA_BIT		) ,
			.AVM_BE_BIT		( AVMM_BE_BIT		) ,
			.AVM_BURST_BIT	( AVMM_BURST_BIT	) ,
			.PKT_SIZE_BIT	( PACKET_SIZE_BIT	) 
		)
		u0_mem_if (
			.clk				( avmm_clk				) ,
			.rst_n				( rst_n					) ,
			.srst				( csr_srst				) ,

			.packet_size_i		( csr_packet_size		) ,

			.fifo_rd_end_o		( mem_if_fifo_rd_end	) ,
			.acc_adr_i			( fifo_bank_acc_adr		) ,
			.fifo_rd_sel_i		( fifo_bank_rd_sel		) ,

			.fifo0_empty_i		( u1_fifo_empty			) ,
			.fifo0_aempty_i		( u1_fifo_aempty		) ,
			.fifo0_rd_o			( u1_fifo_ren			) ,
			.fifo0_rdata_i		( u1_fifo_rdata			) ,
			.fifo1_empty_i		( u2_fifo_empty			) ,
			.fifo1_aempty_i		( u2_fifo_aempty		) ,
			.fifo1_rd_o			( u2_fifo_ren			) ,
			.fifo1_rdata_i		( u2_fifo_rdata			) ,

			.read_o				( avmm_read_o			) ,
			.write_o			( avmm_write_o			) ,
			.waitrequest_i		( avmm_waitrequest_i	) ,
			.address_o			( avmm_address_o		) ,
			.burstcount_o		( avmm_burstcount_o		) ,
			.byteenable_o		( avmm_byteenable_o		) ,
			.writedata_o		( avmm_writedata_o		) ,
			.readdatavalid_i	( avmm_readdatavalid_i	) ,
			.readdata_i			( avmm_readdata_i		) 
		) ;


endmodule

`default_nettype wire
