// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_mem_if.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_mem_if (
	clk				,
	rst_n			,
	srst			,

	packet_size_i	,

	fifo_rd_end_o	,
	acc_adr_i		,
	fifo_rd_sel_i	,

	fifo0_empty_i	,
	fifo0_aempty_i	,
	fifo0_rd_o		,
	fifo0_rdata_i	,
	fifo1_empty_i	,
	fifo1_aempty_i	,
	fifo1_rd_o		,
	fifo1_rdata_i	,

	read_o			,
	write_o			,
	waitrequest_i	,
	address_o		,
	burstcount_o	,
	byteenable_o	,
	writedata_o		,
	readdatavalid_i	,
	readdata_i		
) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	AVM_ADR_BIT		= 27 ;
	parameter	AVM_DATA_BIT	= 128 ;
	parameter	AVM_BE_BIT		= AVM_DATA_BIT / 8 ;
	parameter	AVM_BURST_BIT	= 8 ;
	parameter	PKT_SIZE_BIT	= 11 ;
	localparam	FIFO_BANK_NUM	= 2 ;
	localparam	FIFO_DATA_BIT	= AVM_DATA_BIT ;

	localparam	BURST_COUNT		= ( AVM_DATA_BIT == 256 ) ?  8'd32 :
								  ( AVM_DATA_BIT == 128 ) ?  8'd64 :
								  ( AVM_DATA_BIT ==  64 ) ? 8'd128 : 8'd128 ;

// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire							clk					;
	input	wire							rst_n				;
	input	wire							srst				;

	input	wire	[  PKT_SIZE_BIT-1 : 0 ]	packet_size_i		;

	output	wire							fifo_rd_end_o		;
	input	wire	[ AVM_ADR_BIT+4-1 : 0 ]	acc_adr_i			;
	input	wire	[               1 : 0 ]	fifo_rd_sel_i		;

	input	wire							fifo0_empty_i		;
	input	wire							fifo0_aempty_i		;
	output	wire							fifo0_rd_o			;
	input	wire	[ FIFO_DATA_BIT-1 : 0 ]	fifo0_rdata_i		;
	input	wire							fifo1_empty_i		;
	input	wire							fifo1_aempty_i		;
	output	wire							fifo1_rd_o			;
	input	wire	[ FIFO_DATA_BIT-1 : 0 ]	fifo1_rdata_i		;

	output	wire							read_o				;
	output	wire							write_o				;
	input	wire							waitrequest_i		;
	output	wire	[   AVM_ADR_BIT-1 : 0 ]	address_o			;
	output	wire	[ AVM_BURST_BIT-1 : 0 ]	burstcount_o		;
	output	wire	[    AVM_BE_BIT-1 : 0 ]	byteenable_o		;
	output	wire	[  AVM_DATA_BIT-1 : 0 ]	writedata_o			;
	input	wire							readdatavalid_i		;
	input	wire	[  AVM_DATA_BIT-1 : 0 ]	readdata_i			;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	reg								fifo_rd_sel_ff		;
	wire							rd_start			;
	reg								trans_ff			;
	reg								rd_start_dly1_ff	;
	reg								rd_start_dly2_ff	;
	reg								wt_start_ff			;
	wire							adr_offset			;
	wire							fifo_rd				;
	reg								rdata_valid_ff		;
	reg		[  AVM_DATA_BIT-1 : 0 ]	rdata_lat1_ff		;
	reg		[  AVM_DATA_BIT-1 : 0 ]	rdata_lat2_ff		;
	reg		[  AVM_DATA_BIT-1 : 0 ]	rdata_lat3_ff		;
	reg								write_ff			;
	reg		[ AVM_ADR_BIT+4-1 : 0 ]	address_ff			;
	reg		[ AVM_BURST_BIT-1 : 0 ]	burstcount_ff		;
	reg		[  PKT_SIZE_BIT-1 : 0 ]	packet_size_ff		;
	reg								waitrequest_ff		;
	wire	[    AVM_BE_BIT-1 : 0 ]	first_be			;
	wire	[    AVM_BE_BIT-1 : 0 ]	last_be				;
	wire	[  AVM_DATA_BIT-1 : 0 ]	wdata_sel			;
	reg		[    AVM_BE_BIT-1 : 0 ]	byteenable_ff		;
	reg		[  AVM_DATA_BIT-1 : 0 ]	writedata_ff		;
	reg		[ AVM_BURST_BIT-1 : 0 ]	trans_cnt_ff		;
	reg								last_data_lat1_ff	;
	reg								last_data_lat2_ff	;
	reg								last_data_lat3_ff	;
	wire							last_wt				;
	wire							wt_end				;
	wire							trans_end			;
	reg								trans_lat_ff		;
	reg								fifo_rd_end_ff		;


// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			fifo_rd_sel_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			fifo_rd_sel_ff	<= 1'b0 ;
		end
		else begin
			fifo_rd_sel_ff	<= |fifo_rd_sel_i ;
		end
	end

	assign	rd_start	= ~fifo_rd_sel_ff & |fifo_rd_sel_i ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			trans_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			trans_ff	<= 1'b0 ;
		end
		else begin
			if ( rd_start ) begin
				trans_ff	<= 1'b1 ;
			end
			else if ( trans_end ) begin
				trans_ff	<= 1'b0 ;
			end
		end
	end
	
	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			rd_start_dly1_ff	<= 1'b0 ;
			rd_start_dly2_ff	<= 1'b0 ;
			wt_start_ff			<= 1'b0 ;
		end
		else if ( srst ) begin
			rd_start_dly1_ff	<= 1'b0 ;
			rd_start_dly2_ff	<= 1'b0 ;
			wt_start_ff			<= 1'b0 ;
		end
		else begin
			rd_start_dly1_ff	<= rd_start ;
			rd_start_dly2_ff	<= rd_start_dly1_ff ;
			wt_start_ff			<= rd_start_dly2_ff ;
		end
	end

	assign	adr_offset	= ( AVM_DATA_BIT == 256 ) ?	( |address_ff[4:0] ) :
						  ( AVM_DATA_BIT == 128 ) ?	( |address_ff[3:0] ) :
													( |address_ff[2:0] ) ;

	// fifo control
	assign	fifo_rd	= rd_start_dly1_ff | rd_start_dly2_ff | wt_start_ff | ( write_ff & !waitrequest_i ) ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			rdata_valid_ff	<= 1'b0 ;
			rdata_lat1_ff	<= { AVM_DATA_BIT{1'b0} } ;
			rdata_lat2_ff	<= { AVM_DATA_BIT{1'b0} } ;
			rdata_lat3_ff	<= { AVM_DATA_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			rdata_valid_ff	<= 1'b0 ;
			rdata_lat1_ff	<= { AVM_DATA_BIT{1'b0} } ;
			rdata_lat2_ff	<= { AVM_DATA_BIT{1'b0} } ;
			rdata_lat3_ff	<= { AVM_DATA_BIT{1'b0} } ;
		end
		else begin
			rdata_valid_ff	<= fifo_rd ;
			if ( rdata_valid_ff ) begin
				rdata_lat1_ff	<= ( fifo_rd_sel_i[0] ) ?	fifo0_rdata_i	: fifo1_rdata_i ;
				rdata_lat2_ff	<= rdata_lat1_ff ;
				rdata_lat3_ff	<= rdata_lat2_ff ;
			end
		end
	end


	// Avalon-MM control
	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			write_ff		<= 1'b0 ;
			address_ff		<= { AVM_ADR_BIT+4{1'b0} } ;
			burstcount_ff	<= { AVM_BURST_BIT{1'b0} } ;
			packet_size_ff	<= { PKT_SIZE_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			write_ff		<= 1'b0 ;
			address_ff		<= { AVM_ADR_BIT+4{1'b0} } ;
			burstcount_ff	<= { AVM_BURST_BIT{1'b0} } ;
			packet_size_ff	<= { PKT_SIZE_BIT{1'b0} } ;
		end
		else begin
			if ( rd_start ) begin
				address_ff		<= acc_adr_i ;
				packet_size_ff	<= packet_size_i ;
			end
			if ( wt_start_ff ) begin
				write_ff		<= 1'b1 ;
				burstcount_ff	<= ( adr_offset ) ?	packet_size_ff[PKT_SIZE_BIT-1:4] + 1'b1 :
													packet_size_ff[PKT_SIZE_BIT-1:4] ;
//				burstcount_ff	<= ( adr_offset ) ?	BURST_COUNT + 1'b1 :
//													BURST_COUNT ;
			end
			else if ( trans_end ) begin
				write_ff		<= 1'b0 ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			waitrequest_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			waitrequest_ff	<= 1'b0 ;
		end
		else begin
			waitrequest_ff	<= waitrequest_i ;
		end
	end

	generate
		if ( AVM_BE_BIT == 16 ) begin	: BE_GEN_BE_BIT_16
			assign	first_be	= ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h0 ) } } & 16'hFFFF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h1 ) } } & 16'hFFFE ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h2 ) } } & 16'hFFFC ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h3 ) } } & 16'hFFF8 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h4 ) } } & 16'hFFF0 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h5 ) } } & 16'hFFE0 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h6 ) } } & 16'hFFC0 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h7 ) } } & 16'hFF80 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h8 ) } } & 16'hFF00 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h9 ) } } & 16'hFE00 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hA ) } } & 16'hFC00 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hB ) } } & 16'hF800 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hC ) } } & 16'hF000 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hD ) } } & 16'hE000 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hE ) } } & 16'hC000 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hF ) } } & 16'h8000 ) ;
			assign	last_be		= ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h0 ) } } & 16'hFFFF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h1 ) } } & 16'h7FFF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h2 ) } } & 16'h3FFF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h3 ) } } & 16'h1FFF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h4 ) } } & 16'h0FFF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h5 ) } } & 16'h07FF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h6 ) } } & 16'h03FF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h7 ) } } & 16'h01FF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h8 ) } } & 16'h00FF ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'h9 ) } } & 16'h007F ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hA ) } } & 16'h003F ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hB ) } } & 16'h001F ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hC ) } } & 16'h000F ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hD ) } } & 16'h0007 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hE ) } } & 16'h0003 ) |
								  ( { AVM_BE_BIT{ ( address_ff[3:0] == 4'hF ) } } & 16'h0001 ) ;
			assign	wdata_sel	= ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h0 ) ) } } & { rdata_lat1_ff[127:  0]                          } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h1 ) ) } } & { rdata_lat1_ff[119:  0] , rdata_lat2_ff[127:120] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h2 ) ) } } & { rdata_lat1_ff[111:  0] , rdata_lat2_ff[127:112] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h3 ) ) } } & { rdata_lat1_ff[103:  0] , rdata_lat2_ff[127:104] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h4 ) ) } } & { rdata_lat1_ff[ 95:  0] , rdata_lat2_ff[127: 96] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h5 ) ) } } & { rdata_lat1_ff[ 87:  0] , rdata_lat2_ff[127: 88] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h6 ) ) } } & { rdata_lat1_ff[ 79:  0] , rdata_lat2_ff[127: 80] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h7 ) ) } } & { rdata_lat1_ff[ 71:  0] , rdata_lat2_ff[127: 72] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h8 ) ) } } & { rdata_lat1_ff[ 63:  0] , rdata_lat2_ff[127: 64] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'h9 ) ) } } & { rdata_lat1_ff[ 55:  0] , rdata_lat2_ff[127: 56] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'hA ) ) } } & { rdata_lat1_ff[ 47:  0] , rdata_lat2_ff[127: 48] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'hB ) ) } } & { rdata_lat1_ff[ 39:  0] , rdata_lat2_ff[127: 40] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'hC ) ) } } & { rdata_lat1_ff[ 31:  0] , rdata_lat2_ff[127: 32] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'hD ) ) } } & { rdata_lat1_ff[ 23:  0] , rdata_lat2_ff[127: 24] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'hE ) ) } } & { rdata_lat1_ff[ 15:  0] , rdata_lat2_ff[127: 16] } ) |
								  ( { AVM_DATA_BIT{ ( (  wt_start_ff | ~waitrequest_ff ) & ( address_ff[3:0] == 4'hF ) ) } } & { rdata_lat1_ff[  7:  0] , rdata_lat2_ff[127:  8] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h0 ) ) } } & { rdata_lat2_ff[127:  0]                          } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h1 ) ) } } & { rdata_lat2_ff[119:  0] , rdata_lat3_ff[127:120] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h2 ) ) } } & { rdata_lat2_ff[111:  0] , rdata_lat3_ff[127:112] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h3 ) ) } } & { rdata_lat2_ff[103:  0] , rdata_lat3_ff[127:104] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h4 ) ) } } & { rdata_lat2_ff[ 95:  0] , rdata_lat3_ff[127: 96] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h5 ) ) } } & { rdata_lat2_ff[ 87:  0] , rdata_lat3_ff[127: 88] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h6 ) ) } } & { rdata_lat2_ff[ 79:  0] , rdata_lat3_ff[127: 80] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h7 ) ) } } & { rdata_lat2_ff[ 71:  0] , rdata_lat3_ff[127: 72] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h8 ) ) } } & { rdata_lat2_ff[ 63:  0] , rdata_lat3_ff[127: 64] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'h9 ) ) } } & { rdata_lat2_ff[ 55:  0] , rdata_lat3_ff[127: 56] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'hA ) ) } } & { rdata_lat2_ff[ 47:  0] , rdata_lat3_ff[127: 48] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'hB ) ) } } & { rdata_lat2_ff[ 39:  0] , rdata_lat3_ff[127: 40] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'hC ) ) } } & { rdata_lat2_ff[ 31:  0] , rdata_lat3_ff[127: 32] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'hD ) ) } } & { rdata_lat2_ff[ 23:  0] , rdata_lat3_ff[127: 24] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'hE ) ) } } & { rdata_lat2_ff[ 15:  0] , rdata_lat3_ff[127: 16] } ) |
								  ( { AVM_DATA_BIT{ ( ( ~wt_start_ff &  waitrequest_ff ) & ( address_ff[3:0] == 4'hF ) ) } } & { rdata_lat2_ff[  7:  0] , rdata_lat3_ff[127:  8] } ) ;
		end
	endgenerate

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			byteenable_ff	<= { AVM_BE_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			byteenable_ff	<= { AVM_BE_BIT{1'b0} } ;
		end
		else begin
			if ( wt_end ) begin
				byteenable_ff	<= ( !waitrequest_i ) ?	{ AVM_BE_BIT{1'b0} }	: byteenable_ff ;
			end
			else if ( wt_start_ff ) begin
				if ( last_wt ) begin
					byteenable_ff	<= first_be & last_be ;
				end
				else begin
					byteenable_ff	<= first_be ;
				end
			end
			else if ( last_wt ) begin
				byteenable_ff	<= ( !waitrequest_i ) ?	last_be	: byteenable_ff ;
			end
			else begin
				byteenable_ff	<= ( !waitrequest_i ) ?	{ AVM_BE_BIT{1'b1} }	: byteenable_ff ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			writedata_ff	<= { AVM_DATA_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			writedata_ff	<= { AVM_DATA_BIT{1'b0} } ;
		end
		else begin
			if ( wt_start_ff | !waitrequest_i ) begin
				writedata_ff	<= wdata_sel ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			trans_cnt_ff	<= { AVM_BURST_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			trans_cnt_ff	<= { AVM_BURST_BIT{1'b0} } ;
		end
		else begin
			if ( wt_start_ff ) begin
				trans_cnt_ff	<= ( adr_offset ) ?	packet_size_ff[PKT_SIZE_BIT-1:4] :
													packet_size_ff[PKT_SIZE_BIT-1:4] - 1'b1;
//				trans_cnt_ff	<= ( adr_offset ) ?	BURST_COUNT :
//													BURST_COUNT - 1'b1;
			end
			else if ( trans_cnt_ff == { AVM_BURST_BIT{1'b0} } ) begin
				trans_cnt_ff	<= trans_cnt_ff ;
			end
			else if ( !waitrequest_i && write_ff ) begin
				trans_cnt_ff	<= trans_cnt_ff - 1'b1 ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			last_data_lat1_ff	<= 1'b0 ;
			last_data_lat2_ff	<= 1'b0 ;
			last_data_lat3_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			last_data_lat1_ff	<= 1'b0 ;
			last_data_lat2_ff	<= 1'b0 ;
			last_data_lat3_ff	<= 1'b0 ;
		end
		else begin
			if ( fifo_rd_end_ff ) begin
				last_data_lat1_ff	<= 1'b0 ;
				last_data_lat2_ff	<= 1'b0 ;
				last_data_lat3_ff	<= 1'b0 ;
			end
			else if ( rdata_valid_ff ) begin
				last_data_lat1_ff	<= ( fifo_rd_sel_i[0] & fifo0_empty_i ) | ( fifo_rd_sel_i[1] & fifo1_empty_i ) ;
				last_data_lat2_ff	<= last_data_lat1_ff ;
				last_data_lat3_ff	<= last_data_lat2_ff ;
			end
		end
	end

	assign	last_wt		= ( ~waitrequest_ff & last_data_lat1_ff ) |
						  (  waitrequest_ff & last_data_lat2_ff ) ;
	assign	wt_end		= ( ~waitrequest_ff & last_data_lat2_ff ) |
						  (  waitrequest_ff & last_data_lat3_ff ) ;
	assign	trans_end	= ( trans_cnt_ff == { AVM_BURST_BIT{1'b0} } ) & !waitrequest_i & write_ff ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			trans_lat_ff	<= 1'b0 ;
			fifo_rd_end_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			trans_lat_ff	<= 1'b0 ;
			fifo_rd_end_ff	<= 1'b0 ;
		end
		else begin
			trans_lat_ff	<= trans_ff ;
			fifo_rd_end_ff	<= trans_lat_ff & ~trans_ff ;
		end
	end

	//---------------//
	// output signal //
	//---------------//
	assign	fifo_rd_end_o		= fifo_rd_end_ff ;

	assign	fifo0_rd_o			= fifo_rd_sel_i[0] & fifo_rd ;
	assign	fifo1_rd_o			= fifo_rd_sel_i[1] & fifo_rd ;

	assign	read_o			= 1'b0 ;
	assign	write_o			= write_ff ;
	assign	address_o		= address_ff[AVM_ADR_BIT+4-1:4] ;
	assign	burstcount_o	= burstcount_ff ;
	assign	byteenable_o	= byteenable_ff ;
	assign	writedata_o		= writedata_ff ;


endmodule

`default_nettype wire
