// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_pfifo_rd.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/11/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_pfifo_rd (
	clk				,
	rst_n			,
	srst			,

	fifo_empty_i	,
	fifo_aempty_i	,
	fifo_re_o		,
	fifo_rdata_i	,

	valid_o			,
	startofpacket_o	,
	endofpacket_o	,
	data_o			,
	ready_i			
) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	ST_DATA_BIT		= 64 ;				// Avalon-ST data bit width
	localparam	FIFO_DATA_BIT	= ST_DATA_BIT + 2 ;	// FIFO data bit width


// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire							clk				;
	input	wire							rst_n			;
	input	wire							srst			;

	input	wire							fifo_empty_i	;
	input	wire							fifo_aempty_i	;
	output	wire							fifo_re_o		;
	input	wire	[ FIFO_DATA_BIT-1 : 0 ]	fifo_rdata_i	;

	output	wire							valid_o			;
	output	wire							startofpacket_o	;
	output	wire							endofpacket_o	;
	output	wire	[   ST_DATA_BIT-1 : 0 ]	data_o			;
	input	wire							ready_i			;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	reg								ready_dly_ff	;
	reg								rd_valid_ff		;
	reg								valid_pre_ff	;
	reg								sop_pre_ff		;
	reg								eop_pre_ff		;
	reg		[ ST_DATA_BIT-1 : 0 ]	data_pre_ff		;
	reg								valid_lat_ff	;
	reg								sop_lat_ff		;
	reg								eop_lat_ff		;
	reg		[ ST_DATA_BIT-1 : 0 ]	data_lat_ff		;
	reg								valid_ff		;
	reg								sop_ff			;
	reg								eop_ff			;
	reg		[ ST_DATA_BIT-1 : 0 ]	data_ff			;


// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			ready_dly_ff	<= 1'b0 ;
			rd_valid_ff		<= 1'b0 ;
		end
		else if ( srst ) begin
			ready_dly_ff	<= 1'b0 ;
			rd_valid_ff		<= 1'b0 ;
		end
		else begin
			ready_dly_ff	<= ready_i ;
			rd_valid_ff		<= ready_i & ~fifo_empty_i ;
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			valid_pre_ff	<= 1'b0 ;
			sop_pre_ff		<= 1'b0 ;
			eop_pre_ff		<= 1'b0 ;
			data_pre_ff		<= { ST_DATA_BIT{1'b0} } ;
			valid_lat_ff	<= 1'b0 ;
			sop_lat_ff		<= 1'b0 ;
			eop_lat_ff		<= 1'b0 ;
			data_lat_ff		<= { ST_DATA_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			valid_pre_ff	<= 1'b0 ;
			sop_pre_ff		<= 1'b0 ;
			eop_pre_ff		<= 1'b0 ;
			data_pre_ff		<= { ST_DATA_BIT{1'b0} } ;
			valid_lat_ff	<= 1'b0 ;
			sop_lat_ff		<= 1'b0 ;
			eop_lat_ff		<= 1'b0 ;
			data_lat_ff		<= { ST_DATA_BIT{1'b0} } ;
		end
		else begin
			if ( ready_dly_ff ) begin
				valid_pre_ff	<= rd_valid_ff ;
				sop_pre_ff		<= fifo_rdata_i[FIFO_DATA_BIT-1] ;
				eop_pre_ff		<= fifo_rdata_i[FIFO_DATA_BIT-2] ;
				data_pre_ff		<= fifo_rdata_i[ST_DATA_BIT-1:0] ;
				valid_lat_ff	<= valid_pre_ff ;
				sop_lat_ff		<= sop_pre_ff ;
				eop_lat_ff		<= eop_pre_ff ;
				data_lat_ff		<= data_pre_ff ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			valid_ff	<= 1'b0 ;
			sop_ff		<= 1'b0 ;
			eop_ff		<= 1'b0 ;
			data_ff		<= { ST_DATA_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			valid_ff	<= 1'b0 ;
			sop_ff		<= 1'b0 ;
			eop_ff		<= 1'b0 ;
			data_ff		<= { ST_DATA_BIT{1'b0} } ;
		end
		else begin
			if ( ready_i ) begin
				valid_ff	<= ( ready_dly_ff ) ?	valid_pre_ff	: valid_lat_ff ;
				sop_ff		<= ( ready_dly_ff ) ?	sop_pre_ff		: sop_lat_ff ;
				eop_ff		<= ( ready_dly_ff ) ?	eop_pre_ff		: eop_lat_ff ;
				data_ff		<= ( ready_dly_ff ) ?	data_pre_ff		: data_lat_ff ;
			end
		end
	end


	assign	fifo_re_o		= ready_i ;

	assign	valid_o			= valid_ff ;
	assign	startofpacket_o	= sop_ff ;
	assign	endofpacket_o	= eop_ff ;
	assign	data_o			= data_ff ;


endmodule

`default_nettype wire
